@@ -1,5 +1,5 @@
 package HTTP::API::Client;
-$HTTP::API::Client::VERSION = '1.0';
+$HTTP::API::Client::VERSION = '1.01';
 use strict;
 use warnings;
 
@@ -618,21 +618,69 @@ sub kvp2str {
     my @parts;
 
     foreach my $key(@keys) {
-        if ($o{skip_key}{$key}) {
-            next;
-        }
-
-        my $val = _defor($data->{$key}, '');
-
-        $val = uri_escape($val);
-        $key = uri_escape($key);
-
-        push @parts, "$key=$val";
+        next if $o{skip_key}{$key};
+        push @parts, $self->kvp2str_each(%o, key => $key, value => $data->{$key});
     }
 
     return join '&', @parts;
 }
 
+sub kvp2str_each {
+    my ($self, %o) = @_;
+
+    my ($k, $v) = @o{qw( key value )};
+
+    if (UNIVERSAL::isa($v, 'CODE')) {
+        $v = $self->$v(%o, key => $k);
+    }
+
+    if (!ref $v) {
+        $v = uri_escape($v);
+
+        if ($o{no_key}) {
+            return $v;
+        }
+        else {
+            $k = uri_escape($k);
+            return "$k=$v";
+        }
+    }
+
+    if (ref $v eq 'ARRAY') {
+        my @parts;
+
+        foreach my $val(@$v) {
+            push @parts, $self->kvp2str_each(%o, key => $k, value => $val, no_key => 0);
+        }
+
+        return ($o{no_key} ? '&' : '') . join '&', @parts;
+    }
+
+    if (ref $v eq 'CSV') {
+        my @csv;
+        my @parts;
+
+        foreach my $val(@$v) {
+            my $part = $self->kvp2str_each(%o, key => $k, value => $val, no_key => 1);
+
+            if ($part =~ m/&/) {
+                push @parts, $part;
+            }
+            else {
+                push @csv, $part;
+            }
+        }
+
+        my $csv = "$k=".join( ',', @csv);
+        
+        if (@parts) {
+            return join '&', $csv, @parts;
+        }
+
+        return $csv;
+    }
+}
+
 sub basic_authenticator {
     my ($self, $req, $u, $p) = @_;
     return $req->headers->authorization_basic($u, $p);
